<?php

namespace Modules\JumpSeat\Http\Controllers;

use App\Contracts\Controller;
use App\Models\Aircraft;
use App\Models\Airport;
use App\Models\Pirep;
use App\Models\Enums\FuelType;
use App\Models\Enums\PirepState;
use App\Services\AirportService;
use App\Services\FinanceService;
use App\Support\Math;
use App\Support\Money;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Carbon;
use Lang;
use Log;

class JumpSeatController extends Controller
{
  // Aircraft Transfer
  public function actransfer(Request $request) {
    // Get User Details
    $user = Auth::user();
    $user_location = isset($user->curr_airport_id) ? $user->curr_airport_id : $user->home_airport_id;

    // Get Form Values
    $price = $request->price;
    $current_page = $request->croute;
    $selected_ac = $request->ac_selection;

    // Return If No Aircraft Selected
    if (!$selected_ac) {
      flash(Lang::get('JumpSeat::transferac.errornoac'))->error();

      return redirect(url($current_page));
    }

    // Get Aircraft
    $aircraft = Aircraft::where('id', $selected_ac)->first();

    // Transfer Is Free, Move The Aircraft And Return
    if ($price === 'free') {
      $aircraft->airport_id = $user_location;
      $aircraft->save();
      flash(Lang::get('JumpSeat::transferac.successfree', ['registration' => $aircraft->registration]))->success();
      Log::info('Aircraft Transfer | '.$aircraft->registration.' moved to '.$user_location.' by '.$user->name_private);

      return redirect(url($current_page));
    }

    // Transfer is Auto Price, Define Costs and Continue
    if ($price === 'auto') {
      $ac_location = isset($aircraft->airport_id) ? $aircraft->airport_id : $aircraft->subfleet->hub_id;
      $base_airport = Airport::where('id', $ac_location)->first();
      $dest_airport = Airport::where('id', $user_location)->first();

      // Get Distance Between Airports
      $AirportSvc = app(AirportService::class);
      $transfer_distance = $AirportSvc->calculateDistance($base_airport->id, $dest_airport->id);

      // Get Fuel Price for Departure Airport
      if ($aircraft->subfleet->fuel_type == FuelType::LOW_LEAD) {
        $fuel_price = !empty($base_airport->fuel_100ll_cost) ? $base_airport->fuel_100ll_cost : setting('airports.default_100ll_fuel_cost');
      } elseif ($aircraft->subfleet->fuel_type == FuelType::MOGAS) {
        $fuel_price = !empty($base_airport->fuel_mogas_cost) ? $base_airport->fuel_mogas_cost : setting('airports.default_mogas_fuel_cost');
      } else {
        $fuel_price = !empty($base_airport->fuel_jeta_cost) ? $base_airport->fuel_jeta_cost : setting('airports.default_jet_a_fuel_cost');
      }

      // Get Ground Handling Prices with Subfleet Multiplier
      $orig_gh = !empty($base_airport->ground_handling_cost) ? $base_airport->ground_handling_cost : setting('airports.default_ground_handling_cost');
      $dest_gh = !empty($dest_airport->ground_handling_cost) ? $dest_airport->ground_handling_cost : setting('airports.default_ground_handling_cost');
      $gh_multiplier = filled($aircraft->subfleet->ground_handling_multiplier) ? $aircraft->subfleet->ground_handling_multiplier.'%' : '100%';
      $gh_cost = Math::applyAmountOrPercent(round($orig_gh + $dest_gh, 2), $gh_multiplier);

      // Get Fuel Burn Per Nautical Mile with failsafe (division by zero)
      $subfleet_members = Aircraft::where('subfleet_id', $aircraft->subfleet_id)->pluck('id')->toArray();
      $subfleet_pireps = Pirep::select('distance', 'fuel_used')->whereIn('aircraft_id', $subfleet_members)->where('state', PirepState::ACCEPTED)->get();
      $total_fuelused = $subfleet_pireps->sum('fuel_used');
      $total_distance = $subfleet_pireps->sum('distance');
      if ($total_fuelused > 0 && $total_distance > 0) {
        $avrg_fuelburn = round($total_fuelused / $total_distance, 3);
      } else {
        $avrg_fuelburn = rand(5, 30);
      }
      $aprx_fuelburn = round($avrg_fuelburn * (string)$transfer_distance, 3);
      $fuel_cost = round($aprx_fuelburn * $fuel_price, 3);

      // Build up the Transfer Cost
      $transfer_cost = Money::createFromAmount(round($gh_cost + $fuel_cost, 2));
    }

    // Transfer is Fixed Price, Define Cost and Continue
    if (is_numeric($price)) {
      $transfer_cost = Money::createFromAmount($price);
    }

    // Check User Balance, Return if NOT enough
    if ($transfer_cost > $user->journal->balance) {
      flash(Lang::get('JumpSeat::transferac.errorfunds', ['price' => $transfer_cost]))->error();

      return redirect(url($current_page));
    }

    // Balance Is OK Proceed Transaction
    // Debit From User, Credit To AC Owner Airline
    $financeSvc = app(FinanceService::class);

    $financeSvc->debitFromJournal(
      $user->journal,
      $transfer_cost,
      $user,
      'Aircraft Transfer '.$aircraft->registration,
      'Aircraft Transfer',
      'actransfer',
      Carbon::now()->format('Y-m-d')
    );

    $financeSvc->creditToJournal(
      $aircraft->subfleet->airline->journal,
      $transfer_cost,
      $user,
      'Aircraft Transfer '.$aircraft->registration.' ('.$user->name_private.')',
      'Aircraft Transfer',
      'actransfer',
      Carbon::now()->format('Y-m-d')
    );

    // Proper Flash Messages
    if ($price === 'auto') {
      flash(Lang::get('JumpSeat::transferac.successauto', ['registration' => $aircraft->registration, 'price' => $transfer_cost, 'distance' => $transfer_distance]))->success();
    } else {
      flash(Lang::get('JumpSeat::transferac.successfixed', ['registration' => $aircraft->registration, 'price' => $transfer_cost]))->success();
    }

    // Move Aircraft and Return
    $aircraft->airport_id = $user_location;
    $aircraft->save();
    Log::info('Aircraft Transfer | '.$aircraft->registration.' moved to '.$user_location.' by '.$user->name_private);

    return redirect(url($current_page));
  }

  // JumpSeat Travel
  public function jstravel(Request $request) {
    // Get User Details
    $user = Auth::user();
    $user_location = isset($user->curr_airport_id) ? $user->curr_airport_id : $user->home_airport_id ;

    // Get Form Values
    $price = $request->price;
    $base_price = $request->basep;
    $current_page = $request->croute;
    $new_location = $request->newloc;

    // No Destination Set Or User Is Already At Selected Destination, Return
    if (!$new_location || $new_location == $user_location) {
      flash(Lang::get('JumpSeat::jstravel.errordest'))->error();

      return redirect(url($current_page));
    }

    // Transfer Is Free, Move Pilot And Return
    if ($price === 'free') {
      $user->curr_airport_id = $new_location;
      $user->save();
      flash(Lang::get('JumpSeat::jstravel.successfree', ['location' => $new_location]))->success();

      return redirect(url($current_page));
    }

    // Transfer is Not Free, Calculate Transfer Fee and Continue
    if ($price === 'auto') {
      $AirportSvc = app(AirportService::class);
      $transfer_distance = $AirportSvc->calculateDistance($user_location, $new_location);
      $transfer_cost = $base_price * (string)$transfer_distance;
    }

    // Transfer is Fixed Price, Define Cost and Continue
    if (is_numeric($price)) {
      $transfer_cost = $price;
    }

    $transfer_cost = Money::createFromAmount(ceil($transfer_cost));
    // Check User Balance, Return if not enough for the transfer
    if ($transfer_cost > $user->journal->balance) {
      flash(Lang::get('JumpSeat::jstravel.errorfunds', ['price' => $transfer_cost]))->error();

      return redirect(url($current_page));
    }

    // Balance Is OK Proceed Transaction
    $financeSvc = app(FinanceService::class);

    // Debit Transfer Fee From User
    $financeSvc->debitFromJournal(
      $user->journal,
      $transfer_cost,
      $user,
      'JumpSeat Travel '.$new_location,
      'JumpSeat Travel',
      'jumpseat',
      Carbon::now()->format('Y-m-d')
    );

    // Credit Transfer Fee To Airline
    $financeSvc->creditToJournal(
      $user->airline->journal,
      $transfer_cost,
      $user,
      'JumpSeat Travel ('.$user->name_private.')',
      'JumpSeat Travel',
      'jumpseat',
      Carbon::now()->format('Y-m-d')
    );

    // Proper Flash Messages
    if ($price === 'auto') {
      flash(Lang::get('JumpSeat::jstravel.successauto', ['location' => $new_location, 'price' => $transfer_cost, 'distance' => $transfer_distance]))->success();
    } else {
      flash(Lang::get('JumpSeat::jstravel.successfixed', ['location' => $new_location, 'price' => $transfer_cost]))->success();
    }

    // Move Pilot To Destination
    $user->curr_airport_id = $new_location;
    $user->save();

    return redirect(url($current_page));
  }

  // Admin Page
  public function admin() {
    return view('JumpSeat::admin');
  }
}
